// @flow
// @jsx glam
import React from 'react';
import glam from 'glam';

import { Div, Img } from '../primitives';
import { type PropsWithStyles } from '../types';
import { className } from '../utils';
import { getSource } from './component-helpers';
import componentBaseClassNames from './componentBaseClassNames';

type Props = PropsWithStyles & {
  data: Object,
  isFullscreen: boolean,
  isModal: boolean,
};

export const viewCSS = () => ({
  lineHeight: 0,
  position: 'relative',
  textAlign: 'center',
});

const viewBaseClassName = componentBaseClassNames.View;

const View = (props: Props) => {
  const { data, formatters, getStyles, index, currentIndex, isFullscreen, isModal, imagesLength, PlaceholderComponent } = props;
  const [isInScreen, setIsInScreen] = React.useState(false) ;
  const [loaded, setloaded] = React.useState(false);

  const innerProps = {
    alt: formatters.getAltText({ data, index }),
    src: getSource({ data, isFullscreen }),
  };

  React.useEffect(() => {
    if(currentIndex >= index && !isInScreen && imagesLength) {
      setIsInScreen(true);
    }
  },[currentIndex]);

  const confirmLoaded = React.useCallback(() => {
    setloaded(true);
  }, []);

  return (
    <Div
      css={getStyles(viewBaseClassName, props)}
      className={className(viewBaseClassName, { isFullscreen, isModal })}
    >
      {isInScreen ? (
        <React.Fragment>
          <Img
            {...innerProps}
            className={className('view-image', { isFullscreen, isModal })}
            loading="lazy"
            css={{
              height: 'auto',
              maxHeight: loaded ? '100%' : '0%',
              maxWidth: '100%',
              userSelect: 'none',
              opacity: loaded ? 1 : 0
            }}
            onLoad={confirmLoaded}
          />
          {!loaded ? (
            <PlaceholderComponent />
          ) : null}
      </React.Fragment>
      ) : null}
    </Div>
  );
};

export default View;
